#!/bin/sh

# cu upgrade script

XPRE="cu_upgrade >>"
ROOTFS_FILE="rootfs.tar.xz"
PROGRESS_FILE=$(tty)	# path to progress file
STATUS_FILE="/tmp/upgrade_status" # path to status file
BOARD_TYPE="unknown"

STEPS_TOTAL=4
UPGRADE_STEP=0

usage()
{
cat << EOF
	usage: $0 options

	This script writes cu audio images to hardware

	OPTIONS:
	   -h	   show this help text
	   -r/-i   update rootfs <filename>
	   -p	   path to progress file
EOF
}

print_error()
{
	UPGRADE_STEP=$STEPS_TOTAL
	printf "%s ERROR	step %s/%s - %s\n" "$XPRE" "$UPGRADE_STEP" "$STEPS_TOTAL" "$1" >> $PROGRESS_FILE
	printf "{\"done\":true,\"error\":\"true\"}" > $STATUS_FILE
	exit 1
}

print_progress()
{
	printf "%s PROGRESS step %s/%s - %s\n" "$XPRE" "$UPGRADE_STEP" "$STEPS_TOTAL" "$1" >> $PROGRESS_FILE
	printf "{\"done\":false,\"progress\":\"%s\"}" "$1" > $STATUS_FILE
}

print_progress_done()
{
	printf "%s PROGRESS step %s/%s - %s\n" "$XPRE" "$UPGRADE_STEP" "$STEPS_TOTAL" "$1" >> $PROGRESS_FILE
	printf "{\"done\":true,\"progress\":\"%s\"}" "$1" > $STATUS_FILE
}

upgrade_cu_audio()
{
	STEPS_TOTAL=4
	UPGRADE_STEP=0

	# Note: an update will change the application - number
	# The number is stored to a temporary file (removed on reboot)
	# to make sure we don't accidently update the wrong location
	# in case a second update is done, without rebooting.
	if [ ! -e /tmp/application ]
	then
		fw_printenv -n application > /tmp/application
	fi
	APPLICATION=$(cat /tmp/application)

	# currently first partition?
	if [ $APPLICATION -ne 1 ]; then
		ROOTFS_PARTITION="/dev/mmcblk1p1"
		APPLICATION=1
		ROOTFSLABEL="rootfsA"
	else
		ROOTFS_PARTITION="/dev/mmcblk1p2"
		APPLICATION=2
		ROOTFSLABEL="rootfsB"
	fi;

	if [ -z "$ROOTFS_PARTITION" ]
	then
		print_error "Board is missing the required partition."
		exit 1
	fi

	UPGRADE_STEP="1"
	print_progress "Formatting: ${ROOTFS_PARTITION}"
	mkfs.ext4 -E nodiscard -F -L ${ROOTFSLABEL} ${ROOTFS_PARTITION} &> /dev/null

	UPGRADE_STEP="2"
	print_progress "Installing new rootfs"
	rm -rf /tmp/rootfs
	mkdir -p /tmp/rootfs
	mount -o nodiscard,noatime,nodiratime,data=journal ${ROOTFS_PARTITION} /tmp/rootfs

	tar -xf ${ROOTFS_FILE} -C /tmp/rootfs
	echo "$BOARD_TYPE" > /tmp/rootfs/etc/board
	sync
	umount /tmp/rootfs

	UPGRADE_STEP="3"
	print_progress "Update uboot variables: application $APPLICATION"
	fw_setenv application $APPLICATION
	fw_setenv bootdelay 0
	sync

	UPGRADE_STEP="4"
	print_progress_done "Update finished"
}

upgrade_cu_mme()
{
	STEPS_TOTAL=4
	UPGRADE_STEP=0

	BOOT_PARTITION="/dev/mmcblk0p1"

	# Note: we must update the correct partition
	# Check which partition we are currently using
	ROOTFS_PARTITION=$(sed -n 's/.*root=\([^ ]*\).*/\1/p' /proc/cmdline)

	# switch to other partition
	if [ "$ROOTFS_PARTITION" = "/dev/mmcblk0p2" ]
	then
		ROOTFS_PARTITION="/dev/mmcblk0p3"
		ROOTFSLABEL="rootfsB"
	else
		ROOTFS_PARTITION="/dev/mmcblk0p2"
		ROOTFSLABEL="rootfsA"
	fi

	UPGRADE_STEP="1"
	print_progress "Formatting: ${ROOTFS_PARTITION}"
	mkfs.ext4 -E nodiscard -F -L ${ROOTFSLABEL} ${ROOTFS_PARTITION} &> /dev/null

	UPGRADE_STEP="2"
	print_progress "Installing new rootfs & kernel"
	rm -rf /tmp/rootfs
	mkdir -p /tmp/rootfs
	mount -o nodiscard,noatime,nodiratime,data=journal ${ROOTFS_PARTITION} /tmp/rootfs

	tar -xf ${ROOTFS_FILE} -C /tmp/rootfs
	echo "$BOARD_TYPE" > /tmp/rootfs/etc/board
	sed -i 's/ttymxc4/ttymxc1/g' /tmp/rootfs/etc/inittab
	sync

	UPGRADE_STEP="3"
	print_progress "Update kernel & uboot script"

	rm -rf /tmp/boot
	mkdir -p /tmp/boot
	# note: nodiscard isn't supported in ext2/yocto
	mount -o noatime,nodiratime ${BOOT_PARTITION} /tmp/boot

	cp /tmp/rootfs/boot/zImage /tmp/boot
	cp /tmp/rootfs/boot/*.dtb /tmp/boot

	# there is only 1 boardtype (cu_imx6)
	DTB_FILE=imx6q-televic-cu.dtb

	# note: required for backwards compatibility with yocto
	if [ ! -e /tmp/boot/device.txt ]
	then
		cat << EOF > /tmp/boot/device.txt
cu-imx6
EOF
	fi

	if [ ! -e /tmp/boot/video.txt ]
	then
		cat << EOF > /tmp/boot/video.txt
set vid_dev0 off
set vid_dev1 off
EOF
	fi

	# boot script
	cat << EOF > /tmp/boot/boot.txt
set loadaddr 0x12000000
set fdt_addr 0x18000000
set fdt_file $DTB_FILE
set mmcdev 0
set mmcpart 1
set mmcroot $ROOTFS_PARTITION
set bootargs "console=\${console},\${baudrate} root=\${mmcroot} rootwait ro enable_wait_mode=off quiet"
ext2load mmc \${mmcdev}:\${mmcpart} \${fdt_addr} \${fdt_file}
ext2load mmc \${mmcdev}:\${mmcpart} \${loadaddr} zImage
bootz \${loadaddr} - \${fdt_addr}
EOF

	mkimage -A arm -T script -d /tmp/boot/boot.txt /tmp/boot/boot.scr
	sync

	# remove old files
	rm -rf /tmp/boot/uImage*
	rm -rf /tmp/boot/devicetree*
	rm -rf /tmp/boot/idc-imx6.dtb \
	   /tmp/boot/imx6dl-qmx6.dtb \
	   /tmp/boot/imx6dl-umx6.dtb \
	   /tmp/boot/imx6q-qmx6.dtb \
	   /tmp/boot/imx6q-umx6.dtb

	umount /tmp/boot
	umount /tmp/rootfs
	sync

	UPGRADE_STEP="4"
	print_progress_done "Update finished"
}

while getopts "hr:c:i:p:s:" OPTION
do
	case $OPTION in
		h)
			usage
			exit 0
			;;
		c|s)
			# present for backwards compatibility
			;;
		r|i)
			ROOTFS_FILE=$OPTARG
			;;
		p)
			PROGRESS_FILE=$OPTARG
			;;
		-|*)
			;;
	esac
done

# only install on arm devices - all other cases -> pretent success
if [ "$(uname -m)" != "armv7l" ]
then
	UPGRADE_STEP=$STEPS_TOTAL
	print_progress_done "Done"
	exit 0
fi

print_progress "File: ${ROOTFS_FILE}"

# check board type
if [ -e /etc/board ]
then
	BOARD_TYPE=$(cat /etc/board)
else
	# NOTE: we might have an older (< 5.4) board

	# Create temporary mountpoint
	mkdir -p /tmp/boot
	# Mount first partition
	mount /dev/mmcblk0p1 /tmp/boot

	# check if device.txt exists
	if [ ! -e /tmp/boot/device.txt ]
	then
		umount /tmp/boot

		print_error "Unable to determine board"
		exit 1
	fi

	# Check resolution
	IS_CU=$(cat /tmp/boot/device.txt | grep "cu-imx6" | wc -l)
	if [ "$IS_CU" = "1" ]
	then
		BOARD_TYPE="cu_imx6"
	fi

	umount /tmp/boot
fi

print_progress "Boardtype: $BOARD_TYPE"

if [ "$BOARD_TYPE" = "cu_audio" ] ||
   [ "$BOARD_TYPE" = "cu_audio_golden" ]
then
	BOARD_TYPE="cu_audio"
	upgrade_cu_audio
	exit 0
fi

if [ "$BOARD_TYPE" = "cu_imx6" ]
then
	upgrade_cu_mme
	exit 0
fi

print_error "This upgrade can't be installed on this board."
exit 1
