#!/bin/sh

# mmu upgrade script

XPRE="cu_upgrade >>"
ROOTFS_FILE="rootfs.tar.xz"
PROGRESS_FILE=$(tty)	# path to progress file
STATUS_FILE="/tmp/upgrade_status" # path to status file
BOARD_TYPE="unknown"

STEPS_TOTAL=4
UPGRADE_STEP=0

usage()
{
cat << EOF
	usage: $0 options

	This script writes cu audio images to hardware

	OPTIONS:
	   -h	   show this help text
	   -r/-i   update rootfs <filename>
	   -p	   path to progress file
EOF
}

print_error()
{
	UPGRADE_STEP=$STEPS_TOTAL
	printf "%s ERROR	step %s/%s - %s\n" "$XPRE" "$UPGRADE_STEP" "$STEPS_TOTAL" "$1" >> $PROGRESS_FILE
	printf "{\"done\":true,\"error\":\"true\"}" > $STATUS_FILE
	exit 1
}

print_progress()
{
	printf "%s PROGRESS step %s/%s - %s\n" "$XPRE" "$UPGRADE_STEP" "$STEPS_TOTAL" "$1" >> $PROGRESS_FILE
	printf "{\"done\":false,\"progress\":\"%s\"}" "$1" > $STATUS_FILE
}

print_progress_done()
{
	printf "%s PROGRESS step %s/%s - %s\n" "$XPRE" "$UPGRADE_STEP" "$STEPS_TOTAL" "$1" >> $PROGRESS_FILE
	printf "{\"done\":true,\"progress\":\"%s\"}" "$1" > $STATUS_FILE
}

upgrade_cu_amd()
{
	STEPS_TOTAL=4
	UPGRADE_STEP=0

	# Note: we must update the correct partition
	# Check which partition we are currently using
	ROOTFS_PARTITION=$(rdev | cut -f1 -d' ')

	# switch to other partition
	if [ "$ROOTFS_PARTITION" = "/dev/sda1" ]
	then
		ROOTFS_PARTITION="/dev/sda2"
		ROOTFSLABEL="rootfsB"
	else
		ROOTFS_PARTITION="/dev/sda1"
		ROOTFSLABEL="rootfsA"
	fi

	UPGRADE_STEP="1"
	print_progress "Formatting: ${ROOTFS_PARTITION}"
	mkfs.ext4 -E nodiscard -F -L ${ROOTFSLABEL} ${ROOTFS_PARTITION} &> /dev/null

	UPGRADE_STEP="2"
	print_progress "Installing new rootfs & kernel"
	rm -rf /tmp/rootfs
	mkdir -p /tmp/rootfs
	mount -o nodiscard,noatime,nodiratime,data=journal ${ROOTFS_PARTITION} /tmp/rootfs

	tar -xf ${ROOTFS_FILE} -C /tmp/rootfs
	sync

	UPGRADE_STEP="3"
	print_progress "Update bootloader"

	# kernel is always bzImage
	cat << EOF >> /tmp/rootfs/boot/lilo.conf
map=/tmp/rootfs/boot/map
install=/tmp/rootfs/boot/boot.b
default=linux
compact
lba32

image=/tmp/rootfs/boot/bzImage
label=linux
root=${ROOTFS_PARTITION}
append="rootdelay=3 rootwait panic=1"
EOF
	sync

	# yocto doesn't contain a lilo. -> add path to find a version.
	export PATH=$PATH:/tmp/rootfs/usr/bin
	lilo.static -C /tmp/rootfs/boot/lilo.conf -b /dev/sda
	sync

	umount ${ROOTFS_PARTITION}

	UPGRADE_STEP="4"
	print_progress_done "Update finished"
}

while getopts "hr:c:i:p:s:" OPTION
do
	case $OPTION in
		h)
			usage
			exit 0
			;;
		c|s)
			# present for backwards compatibility
			;;
		r|i)
			ROOTFS_FILE=$OPTARG
			;;
		p)
			PROGRESS_FILE=$OPTARG
			;;
		-|*)
			;;
	esac
done

# only install on x86_64 devices - all other cases -> pretent success
if [ "$(uname -m)" != "x86_64" ]
then
	UPGRADE_STEP=$STEPS_TOTAL
	print_progress_done "Done"
	exit 0
fi

print_progress "File: ${ROOTFS_FILE}"

# amd's are always cu mme's
BOARD_TYPE="cu_amd"

if [ "$BOARD_TYPE" = "cu_amd" ]
then
	upgrade_cu_amd
	exit 0
fi

print_error "This upgrade can't be installed on this board."
exit 1
